/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guiaddaccount.h"
#include <qlineedit.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <string.h>

/**
 * this is the addaccountbade constructor
 *
 *@param parent the parent dialog of this widget
 *@param stack
 *@param name widgets title
 *@param hobit
 */

GuiAddAccount::GuiAddAccount( QWidget* parent, GuiMain * guimain, const char *name, HoBIT* hobit)
    :GuiAddAccountBase(parent, name)
{
  _hobit = hobit;
  _main = guimain;

  //connect( lineEditAccountId, SIGNAL( textChanged(QString) ), this, SLOT( validateInput() ) );

}

/** destructor*/
GuiAddAccount::~GuiAddAccount(){
qDebug("Destructor GuiAddAccount called");
}


/**
*After the user typed in the information of the account to be added and pressed the button this function
*checks if at least a accountid is there and then read in the necessary informations for add the account
*/
void GuiAddAccount::accept()
{
  if (lineEditAccountId->text().isEmpty())
  {
    QMessageBox::critical(this, tr("Input Error"),
    ("You have to specify one account at least."));
    return;
  }

  string accountId = lineEditAccountId->text().ascii();
  string accountSubId = lineEditAccountSubId->text().ascii();
  string name = lineEditName->text().ascii();
  string accountName = lineEditAccountName->text().ascii();

  if( _hobit->addAccount(name, accountId, accountSubId, accountName)== HobitErrorSuccess){;
   QMessageBox::information(this, tr("Information"), 
                                  tr("The new account has been added to the account list."));
   emit accountAdded();
  }
  else QMessageBox::critical(this,tr("Error"), tr("Your account could not be created. See log file for details."));
}


/**
*return to last widget that was visible
*/
void GuiAddAccount::slotBack()
{
  _main->slotShowLastVisibleWidget();
}

/**
*validation for informations on account to be added.
*/
void GuiAddAccount::validateInput()
{

  bool ok;
   QString accountId = lineEditAccountId->text();
   QString accountSubId = lineEditAccountSubId->text();

    // if validation is true current input get saved to var. var is used for check next time the text get changed
   ok = _hobit-> validateAccountIdString(accountId);
   if(ok) 
     _lastInputAccountId = accountId;
   else lineEditAccountId->setText(_lastInputAccountId);
   
   if(ok)
     ok = _hobit-> validateAccountIdString(accountSubId);
   if(ok) 
     _lastInputAccountSubId = accountSubId;
   else lineEditAccountSubId->setText(_lastInputAccountSubId);
  
   if (!ok)   
     QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n Only digits are allowed here. \nMaximum is 15 digits."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
}

