/***************************************************************************
                          guidatedsingletransfer.cpp  -  description
                             -------------------
    begin                : Sa Nov 13 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "globals.h"
#include "guidatedsingletransfer.h"
#include <qdatetimeedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlineedit.h>

/**
*constructor of GuiSingleTransfer
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiDatedSingleTransfer::GuiDatedSingleTransfer(QWidget *parent, const char *name, HoBIT* hobit)
:GuiTransferForm(parent, name, hobit) {
   // hide not needed input fields
   formNameLabel->setText(tr("Dated Transfer"));
   _ReconfirmationHeader = tr("You are about to create the following dated transfer: ");
   textLabelLast->hide();
   dateEditLast->hide();
   textLabelTurnus->hide();
   comboBoxTurnus->hide();
   textLabelDay->hide();
   spinBoxDay->hide();
   _FormType = DATED_SINGLE_TRANSFER_WIDGET; 

   refresh();
}

/**
*destructor of GuiSingleTransfer
*/
GuiDatedSingleTransfer::~GuiDatedSingleTransfer(){
	qDebug("Destructor GuiDatedSingleTransfer finished");
}

/** 
* called function after pressing button of formular,
* gets current inputs of formular for transferjob  
*/
void GuiDatedSingleTransfer::accept(){
  _FirstDate = dateEditFirst->date();
  GuiTransferForm::accept();
}

/** Builds the html table rows for the reconfirmation dialog. Returns a string that ist to be included into <table> </table tags. 
*/
QString GuiDatedSingleTransfer::reconfirmationTableRows(){
  QString result = GuiTransferForm::reconfirmationTableRows();
  QDate d = dateEditFirst->date();
  result += "<tr>";
  result += "</tr>\n";

  result += "<tr>";
  result += "<td> <b>" +tr("Date of execution:") + "</b></td>";
  result += "<td><b>" + d.toString(Qt::LocalDate) +  "</td></b>";
  result += "</tr>\n";
  return result;
}

/** Creates the dated single transfer job and returns true on success.
  */
 bool GuiDatedSingleTransfer::createJob(){
  list <string> otherName;
  otherName.push_back(_OtherName);
  list<string> purpose;
  Date date(_FirstDate.day(), _FirstDate.month(), _FirstDate.year());
  HoBIT_String::copyQStringList(&_PurposeList, & purpose);

  bool jobSupported =  HoBIT_Bank::supportsJob(_hobit->getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HITUBS")
                     && HoBIT_Bank::supportsJob(_hobit->getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HITUES");      
  if(! jobSupported){
     QMessageBox::critical(this, tr("Error"), tr("Sorry, your bank does not support dated transfers."));
     return false;
  } 
  
  int err = _hobit->createJobSingleTransfer(  SKIP_FALSE_TRANSACTIONS,      // skipbad
                                              MONEYPENNY_DEFAULT_COUNTRY,                          // country
                                              "*",                            // bankcode
                                              _OurAccount.ascii(),      // accountid
                                              "",                            // accountSubId
                                              MONEYPENNY_DEFAULT_COUNTRY,                          // other country
                                              _OtherBankCode.ascii(),       // other bankcode
                                              _OtherAccountId.ascii(),        // other account
                                              otherName,                    // other name
                                              purpose,                    // purpose
                                              _Value,                           // value
                                              51,                            // textkey
                                              Date());          // use today as date


   if(err != HobitErrorSuccess){
     QMessageBox::critical(this, tr("Error"), tr("Creating the job failed."));
     qDebug("GuiDatedSingleTransfer::accept: Error creating JobDatedSingleTransfer. Error Code: %i", err);
     return false;  
   }
  
   _hobit->createJobGetDatedTransfers(); //immediately afterwards get the new list
   return true; 
 }   
