/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, nc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUIHBCIWIZARD_H
#define GUIHBCIWIZARD_H

//#include "guimain.h"
#include <qwidgetstack.h>
#include "hobit.h"
#include "guihbciwizardbase.h"
class GuiMain;


/**
* class for the wizard to set up the hbci (account-, bank- and user-details and internet connection)
*/

class GuiHbciWizard: public GuiHbciWizardBase
{
  Q_OBJECT
public:

  /**
  *this is the constructor of dialog for setting up wizard  
  *
  *@param main 
  *@param parent the parent dialog of this widget
  *@param name widgets title
  *@param hobit
  */
  GuiHbciWizard( QWidget * parent, const char *name, HoBIT* hobit);

  /** destructor */
  ~GuiHbciWizard();

   /** Creates a html formatted QString containing the data input.
   */
   QString getDataString();
   /** Sets the corresponding radiobutton.(Floppy or Usb) default is floppy.
   */
   void setStorageType(StorageType storage);
   /** Write property for _MediumDir
   */
   void setMediumDir(const QString & dir);
   
  /** reads the user's previous input */
  virtual void evaluateStoredData();

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }

  /** update account list */
  void updateAccountList();
  
  /** Changes the password for the hbci communication. Only valid for RDH files.
  */
  void changePassword();

public slots:
  /**
  * when user typed in bankid, look what bank it is and show it with name and code in lineedit
  */
  virtual void slotBankSelected();
  
  /**
   *function for a test connection to the bank, the user gets informations about the status 
   */
  virtual void slotChangeCustomer();
  /** 
   *Copies text from UserId to CustId
   */
  void slotCopyUserId();
  /**
  * function for check if typed-in bank is existing
  */
  virtual void slotSearchBankId();
  
  /**
   *function for slot new medium
   */
  virtual void slotNewMedium(int state);
  
  /**
   *function for slot existing medium
   */
  virtual void slotExistingMedium(int state);
  
  /**
   *function for slot chip card  
   */
  virtual void slotChipCard(int state);
  
  /**
   *function for change the medium type 
   */
  virtual void slotChangeMediumType();
  
  /**
   *function for choose keyfiles 
   */
  virtual void slotSearchFile();
  
  /**
   *function for a test connection to the bank, the user gets informations about the status 
   */
  virtual void slotTestConnection();

  /**
   * Dialog for send the keys. User gets informations about the status of sending
   */
  virtual void slotSendKeys();

  /** Fingerprints have been checked
  */
  void slotFingerprintChecked();

  /**
   * Executed when the page changes 
   */
  virtual void pageChanged();

  /** Show the help page on password security
  */
  void slotHelpPassword();

  /** Checks the Internet setup.
  */
  void slotTestInternetSetup();
  
  /** Calls Internet setup program.
  */
  void slotInternetSetup();
    
  /** Reads out /etc/chipcard.conf and lists up all found card readers */
  virtual void slotFindCardReaders();
  

protected slots:
  virtual void next()
  {
    GuiHbciWizardBase::next();
    pageChanged();
  }

  virtual void back()
  {
    GuiHbciWizardBase::back();
    pageChanged();
  }

  /**
   * Executed when the user presses cancel button
   */
  virtual void reject();

  /**
   * Executed when the user presses the accept button
   */
  virtual void accept();

  /**
  * Call help widget passing the appropriate help file.
  */
  void help();
  
  /** Sets password bar according to log of number of possible passwords.
  */
  void slotPasswordStrength();
  
  /** Checks password and creates a new key if ok
  */
  void slotCheckPassword();
  /** Updates the state of the controls on bank data page.
  */
  void slotUpdateBankDataPage();
  
  /** show log widget
  */
  void slotShowDetails();
  
  /** configure card reader 
  */
  void configureCardReader();


  /** set up account list 
  */
 bool findAccounts();


  /**
  *raises the add account widget 
  */
  void slotAddAccount();
  
  /** Fires signal dialIn.
  */
  void slotDialIn();
  
signals:
  void testInternet(const QString & url, int port, ConnectionState * state);
  void helpRequest(const QString & helpfile);
  void setupInternet();
  void setupSuccessful(HBCI_MODE RDHMode);
  void showLog(int Page);
  void renewHoBIT();
  void addAccount();
  void dialIn();
  void wizardPageChanged(int Page);
  void aborted();
  
protected:
  /** Searches for an existing key file or asks the user for a name.
  */
  void getKeyFileName();
  
  /**Reads the keyfile an puts data into edits.
  */
  bool evaluateKeyFile(const QString & fileName);
  
  /** Create a new keyfile.
  */
  void createNewKeyfile();
  
  /** Changes password of current keyfile.
  */
  bool changeKeyfilePassword();  
  
  /** Sets hbci combo.
  */
  void setHbciComboBox(int hbciVersion);
  
private:  
  QWidget* _parent;;
  QWidgetStack* _stack;
  QTimer* _timerFindCardReader;
  
  HoBIT* _hobit;
  list<HoBIT_Bank> _banks;

  //Flags
  bool _connectionTestSuccessful;
  bool _KeyFound;
  bool _MediumChanged;
  bool _sendingKeysSuccessful;
  bool _NewKeyCreated; // medium has been created
  bool _ChangingPassword;
  /** The initial directory for key and config
  */
  QString _MediumDir;
  /**The name of the cardreader as detected by chipcard-tool list
  */
  QString _CardReaderType;
  /** The number of the chipcard as a string
  */
  QString _CardNumber;
  /** The HBCIVersion of the bank server.
  */
  int _HBCIVersion;
};

#endif
