/***************************************************************************
                          guimain.h  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUIMAIN_H
#define GUIMAIN_H

#include <qwidget.h>
#include <qwidgetstack.h>
#include <qlistview.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qmenubar.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qtable.h>
#include <qsqlquery.h>
#include <qprocess.h>
#include <qvaluestack.h>


#include <openhbci2/pointer.h>
#include <openhbci2/date.h>
using namespace HBCI;

#include "guimainbase.h"
#include "globals.h"
#include "standingorder.h"
#include "transaction.h"


//class Pointer<class T>;

class GuiBalances;
class GuiBurn;
class GuiSplash;
class GuiTransactions;
class GuiGetTransactions;
class GuiTransactionPrinter;
class GuiTransactionInfo;
class GuiTools;
class GuiShowData;
class GuiTransferTemplates;
class GuiSingleTransfer;
class GuiDatedTransfers;
class GuiStandingOrders;
class GuiStartStandingOrder;
class GuiDatedSingleTransfer;
class GuiHelpWindow;
class GuiIniLetter;
class GuiKnownBanks;
class GuiLogin;
class GuiLog;
class GuiAccountInformation;
class GuiHbciWizard;
class GuiChangeAccountName;
class GuiTransactionInfo;
class GuiJobs;
class GuiAddAccount;
class GuiHardwareConfig;
class GuiDialIn;
class GuiIniLetter;
class GuiTransferTemplates;
class GuiWelcome;
class GuiLoginPassword;
class GuiFindMedium;
class GuiDataExport;
class GuiSqlQuery;
class GuiSqlQueryResult;
class GuiPrinterAutoSetup;
class GuiPrinterManualSetup;
class GuiPrinterDriverToXml;

class HoBIT;
class HoBIT_Connector;
class HoBIT_Crypt;
class HoBIT_SQL;
class Account;
class TransactionItem;
/**
*This is MoneyPenny Main Dialog
*/
class GuiMain : public GuiMainBase
{
  Q_OBJECT

public:

  /**
  *constructor for GuiMain (main window),
  *sets statusbar, menulist, refreshes informations, widgets, slots, etc. 
  *@param crypt pointer on the HoBIT_Crypt interface
  *@param parent pointer on current QWidget
  *@param name title of the GuiMain dialog
  */
  GuiMain();

  /** starts building the main application -- real constructor */
  virtual void buildMainApplication();

  /**
  *destructor of GuiMain
  */
  ~GuiMain();

  /** Opens a dialog which asks the user wether the stored jobs should
  * be send or should not. If the user answers "Yes" the Job Queue will
  * be executed. Otherwise The jobs will be stored for later use.
  */
  int askForSend();
  
  /**
  *Building Menu by adding items and setting labels
  */
  void createMenu();

  /**
  *open slot after pressing an item of menu
  */
  void slotSelectItemFromMenu( QListViewItem * item );

  /**
  *refresh account-balance
  */
  void changeBalanceStatus();

  /**
  *Raise widget with list of known banks
  */
  void slotListKnownBanks();

  /**
  *Switch Account by raise login-widget
  */
  void switchAccount();

  /**
  *Change Accoutname
  */
  void changeAccountName();

  /** Loads the configuration files. 
  */
  bool loadEnvironment();
  
  /**
  *function for closing
  */
  void closeEvent(QCloseEvent * ce);
  
  /** Read property of HoBIT_Connector m_Connector. 
  */
  HoBIT_Connector * getConnector()const;

  /** Read property of QString _CryptDir. 
  */
  const QString& getCryptDir();
  
  /** Write property of QString _CryptDir. This should be on the removable medium.
  */
  void setCryptDir( const QString& _newVal);
  
  /** Read property of QString _ConfigDir. This should be on the ramdisk.
  */
  const QString& getConfigDataDir()const;
  
  /** Write property of QString _ConfigDir.  This should be on the ramdisk.
  */
  void setConfigDataDir( const QString& _newVal);
  
  /** Write property of Storage _Storage. 
  */
  void setStorage( const StorageType& _newVal);
  
  /** Read property of QString _MountPoint. 
  */
  const QString& GuiMain::getMountPoint()const;
  
  /** Write property of QString _MountPoint. 
  */
  void GuiMain::setMountPoint( const QString& _newVal);

  /** shows the splash screen */
  void showSplashScreen();

  /** hides the splash screen */
  void hideSplashScreen();
  
  /** returns the splash screen */
  GuiSplash * getSplashScreen();
    
  /**Search for the removable medium. Returns true on success.*/
  bool findMedium();
  
  /**ask the user for the local data password.*/
  void login();
  

protected:  
  
  /** Called after the correct paths have been set. Be shure to call setConfigDir prior to calling show().
  *   Gets account list.
  */
  void showEvent(QShowEvent * se);
  /** Output of window size
  */
  void resizeEvent (QResizeEvent * re);
  /** Decrypts the data on the removeable medium and copies them to /home/moneypenny_data
  */
  bool decryptData();
  /** Crypts the data in /home/moneypenny_data and copies them to the removeable medium.
  */
  bool encryptData();

  /** Asks the user, whether he wants to close withouts saving.
  */
  void reject();
  /**Create main configuration directory
  */
  void createConfigDir();
  
private:

  // widgets
  GuiTools* _guitools;
  GuiShowData * _GuiShowData;
  GuiWelcome* _guiwelcome;
  //GuiBalances* _guibalances;
  GuiAccountInformation* _guiaccountinformation;
  GuiTransactions* _guitransactions;
  GuiTransactionPrinter* _guitransactionprinter;
  GuiGetTransactions* _guigettransactions;
  GuiIniLetter* _guiiniletter;
  GuiSingleTransfer* _guisingletransfer;
  GuiDatedSingleTransfer* _guidatedsingletransfer;
  GuiDatedTransfers* _guidatedtransfers;
  GuiStandingOrders* _guistandingorders;
  GuiStartStandingOrder* _guistartstandingorder;
  GuiHelpWindow * _guihelpwindow;
  GuiKnownBanks * _guiknownbanks;
  GuiLogin* _guilogin;
  QWidget* _guiNullWidget; // a dummy widget for debugging
  GuiHbciWizard* _guihbciwizard;
  GuiChangeAccountName* _guichangeaccountname;
  GuiJobs* _guijobs;
  GuiAddAccount* _guiaddaccount;
  GuiHardwareConfig* _guihardwareconfig;
  GuiDialIn* _guidialin;
  GuiLog * _guilog;
  GuiTransferTemplates* _guitransfertemplates;
  GuiTransactionInfo* _guitransactioninfo;
  GuiBurn* _guiburn;			// delete after cebit
  GuiSplash * _guiSplash;
  GuiFindMedium * _guiFindMedium;
  GuiLoginPassword * _guiLoginPassword;
  GuiDataExport* _guidataexport;
  GuiSqlQuery* _guisqlquery;
  GuiSqlQueryResult* _guisqlqueryresult;
  GuiPrinterDriverToXml * _guiprinterdrivertoxml;
  GuiPrinterAutoSetup * _guiprinterautosetup;
  GuiPrinterManualSetup * _guiprintermanualsetup;
  
  
  // List view Items
  QListViewItem* _paymentItem;
  QListViewItem* _jobItem;
  QListViewItem* _configItem;
  QListViewItem* _connectItem;

  QListViewItem* _accountItem;
  QListViewItem* _changeBankItem;  
  QListViewItem* _AddBankItem;  
  QListViewItem* _updateAccountList;
  QListViewItem* _selectAccountItem;
  QListViewItem* _accountBalanceItem;
  QListViewItem* _turnOverItem;
  QListViewItem* _getTransactionsItem;
  QListViewItem* _showTransactionsItem;
  QListViewItem* _printTransactionsItem;
  QListViewItem* _exportTransactionsItem;
  QListViewItem* _singleTransferItem;
  QListViewItem* _datedTransferItem;
  QListViewItem* _transferTemplateItem;
  QListViewItem* _standingOrderItem;
  QListViewItem* _sendNowItem;
  QListViewItem* _hbciItem;
  QListViewItem* _ChangeLoginPasswordItem;
  QListViewItem* _ChangeHbciPasswordItem;
  QListViewItem* _CreateBackupMediumItem;
  QListViewItem* _ToolboxItem;
  QListViewItem* _ShowDataItem;
  QListViewItem* _hardwareItem;
  QListViewItem* _helpItem;
  QListViewItem* _updateItem;
  QListViewItem* _exitItem;
  QListViewItem* _knownBanksItem;
  QListViewItem* _jobQueueItem;
    
  QListViewItem* _cebitItem;		// delete after cebit
  QListViewItem* _burnItem;		// delete after cebit

  QTimer* _timer;

  HoBIT_Connector* _connector;
  HoBIT_Crypt* _Crypt;
  HoBIT_SQL* _sql;

 /* list<Pointer<Account> > _accounts;
  list<Pointer<Account> >::iterator _accit;
  int _accountCounter;
*/
  // menu
  QMenuBar* bar;
  QPopupMenu* bankingMenu;
  QAction* balanceAction;
  
  // hobit
  HoBIT* _hobit;

public slots: // Public slots 
  
  /**
  *Raise AccountLogin-Dialog or HBCI-Wizard
  */
  void slotAccountLogin();
  /** Stores the knoppix configuration.
  */
  bool slotStoreOsConfig();

  /** calls the knoppix restore script.
  */
  int slotRestoreOsConfig();
  /**
  *refresh informations, lists and widgets
  */
  void slotRefresh();
  /**
  *Raises a transfer template
  */
  void slotRaiseTemplate( int id, 
                          QString name, 
                          QString accountId, 
                          QString bankId, 
                          QString value, 
                          QStringList purpose);

  
   /** deletes the old hobit object and creates a new one */
   void renewHoBIT();


  /** shows data export dialog */
  void slotExport(QTable* table);
  
  /**
  *Raise widget for ini-letter
  */
  void slotShowIniLetter();
  /**
  *Raises the toolbox-widget on the WidgetStack
  */
  void slotToolbox();
  /**
  *Raises the ShowData-widget on the WidgetStack
  */
  void slotShowData();
  /**
  *Raises the Welcome-widget on the WidgetStack and shows 
  *update informations.
  */
  void slotUpdateProgram();

  /**
  *Raises the HBCI-Widget on the WidgetStack
  */
  void slotHbciSetup();

  /**
  *Raises the Widget, which shows quick information about all hbci accounts
  */
  void slotAccounts();

  /** Get a new account list from bank server.
  */
  void slotUpdateAccountList();
  
   /**
  *Asks for execution of the job list and sends if requested.
  */
  void slotSend();
  
  /**
  *starts execution of the job list 
  */
  void slotSendNow();

  /**
  *Select the accounts for which the transactionlists are to be updated 
  */
  void slotGetTransactions();

  /**
  *shows a dialog with the turnovers 
  */
  void slotShowTransactions();

  /**
  *creates a single transfer 
  */
  void slotSingleTransfer();

  /**
  *calls a transfer template. This is a user defined mask for frequently used transfers. 
  */
  void slotTransferTemplate();

  /**
  *shows dated transfers widget 
  */
  void slotDatedTransfer();

  /**
  * Performed after Setup Wizard has finished
  */
  void slotSetupSuccessful(HBCI_MODE mode);
  
  /**
  *No descriptions 
  */
  void slotStandingOrder();

  /**
  *No descriptions 
  */
  void slotUpdate();

  /**
  *Call slotHelp(QString&)
  */
  void slotHelp();
  /**
  *Open the help window
  *@param helpfile a helping QString
  */
  void slotHelp(const QString & helpfile);

  /**
  *timer done 
  */
  void slotTimerDone();

  /**
  *Prints the transactions.
  */
  void slotPrintTransactions();

  /**
  *Shows the job list
  */
  void slotJobs();

  /**
  *Raise hardware widget in CONNECT mode 
  */
  void slotDialIn();

  /**
  *raises the add account widget 
  */
  void slotAddAccount();
  
  /** Deletes the account accountId.
  */
  void slotDeleteAccount(QString accountId);

  /**
  *raises the hw config dialog 
  */
  void slotHardwareConfig();

  /**
  *shows welcome widget 
  */
  void slotShowWelcome(const QString & smallHelpFile);
  
  /**
  *closes the application 
  */
  void slotClose();
  /**
  * Change bank by shutting down and restarting with new medium 
  */
  void slotChangeBank();

  /**Shuts down current session and runs the select-bank loop.
  */
  void slotHbciSetupAborted();

  /**
  *set account to be used from moneypenny after login 
  *@param ca QString for the current account
  */
  void setCurrentAccount(QString &ca);

  /**
  *get the account that is used by moneypenny 
  */
  QString getCurrentAccount();

   /**
  *Raises widget with given id
  *Builds the name of the small help file from widget name
  *@param id id of the widget which should be raised 
  */
  void slotRaiseWidget(int id);
  
  /**
  *Deactivate Buttons and List
  */
  void slotDeactivateMenuFunctions();

  /**
  *Activate Buttons and List
  */
  void slotActivateMenuFunctions();
  
  /** Adds a log text to the log text widget
  */
  void slotAddLogText(QString logText);

  /** Run a test connection to www.google.de or another url
  */
  void slotTestInternet(const QString & url, int port, ConnectionState * state);
  
  /**
  *Show last visible widget
  */
  void slotShowLastVisibleWidget();
  
  
  /**
  *Restart the Application
  */
  void slotRestart();

 /**
  *shows a transaction information dialog
  */
  void slotShowTransactionInfo(Pointer <Transaction> trc);

   /**
  *shows a transaction information dialog
  */
  void slotShowStandingOrderInfo(Pointer<StandingOrder> sto);
  
  /** shows information about a single dated transaction
  */
  void slotShowDatedTransferInfo(Pointer<DatedTransfer> dt);
  
  /** Import an RDH2 formatted key. 
  */
  void slotImportRDH2Key(const QString & fileName);
  
  /** Shows a dialog to change the login password.
  */
  void slotChangeLoginPassword();

  /** Shows a dialog to change the HBCI password.
  */
  void slotChangeHbciPassword();
  
  /** Shows a dialog to create a backup medium.
  */
  void slotCreateBackupMedium();

  /** Shows a dialog to send an sql query.
  */
  void slotSqlQuery();

  /** Exectutes the sqlquery.
  */
  void slotExecuteSqlQuery(QString query);

  /** Sets Sets _AutoShutdown Mode. 
  */
  void slotSetShutdownMode(bool autoShutdown);
  
  /** Calls the printer setup program
  */
  void slotAddPrinter();

  /** generate sql database */
  void slotMakeSQL(QString filename);
  
  /** Sets the timeOut value in _Connector
  */
  void slotSetTimeOut(int newTimeOut);
  
  /** Raises the printer driver setup widget.
  */
  void slotBuildPrinterDriverDatabase();
  /** Displays helpFile in the small help widget at the lower left.
  */
  void slotShowSmallHelp(const QString & helpFile);
  /**Deletes OpenHbci-LogFiles
  */
  void slotDeleteLogFiles();
  /** Updates _CurrentWizardPage and contents of small help widget
  */
  void slotWizardPageChanged(int newPage);
  /**Shuts down current session and calls HBCI wizard to setup a new bank.
  */
  void slotAddBank();
private:

  /**
  *set Menubar and insert accountbalance
  */
  void doMenu();
  /** Starts the cups printer daemon.
  */
  bool startCupsd();

   /** Storage is the storage type for the config directory (USB or floppy)
  */
  StorageType _Storage;
  /** mount point of the removable device
  */
  QString _MountPoint;
  
  /** Exit Code of the Application 
  */
  ExitCode _exitCode;
  /** Flag for automatic OS-shutdown on Knoppix CD
  */
  bool _AutoShutdown;
  /** Flag for saving Knoppix config
  */
  bool _SaveKnoppixConfig;
  /** The directory for moneypenny's temporary data files.
  */
  QString _ConfigDataDir; 
  /** The directory for moneypenny's encrypted data files.
  */
  QString _CryptDir; 
  /** The printer setup process
  */
  QProcess * _AddPrinterProcess;
  /** The stack with the widgets theat have already been called.
  */
  QValueStack<int> * _WidgetCallStack;
  /**The currently displayed small help file
  */
  QString _SmallHelp;
  /** The present page of the HBCI Wizard
  */
  int _CurrentWizardPage;

 };



#endif
