/***************************************************************************
                          guitools.h  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUITOOLS_H
#define GUITOOLS_H

#include <qwidget.h>
#include <qstring.h>
#include <guitoolsbase.h>
#include "guicreateuser.h"
#include "guimain.h"
#include "guihbciversion.h"
#include "qwidgetstack.h"
#include <qprocess.h>
//#include "guiiniletter.h"
#include "hobit.h"

#include <list>
using namespace std;

#include <openhbci2.h>
using namespace HBCI;


class GuiMain;
class GuiCreateUser;
class GuiHbciVersion;

/**
class for dialogs to create new user or change the current hbci-version
*/
class GuiTools : public GuiToolsBase
{
  Q_OBJECT
public:

  /**
  *this is the constructor for create new user
  *
  *@param main
  *@param parent the parent dialog of this widget
  *@param name widgets title
  *@param hobit
  */
  GuiTools(QWidget *parent, const char *name=0, HoBIT* hobit = 0);
  ~GuiTools();

/** set hobit object*/
  void setHoBIT(HoBIT* hobit);

   /** Updates the data listview.
  */
  void refresh();
  /** Sets the timeOut value in lineEditTimeOut
  */
  void setTimeOut(int newTimeOut);
  
signals:
 void testInternet(const QString & url, int port, ConnectionState * state);
 void helpRequest(const QString& );
 void importRDH2Key(const QString & fileName);
 void sqlquery();
 void setShutdownMode(bool autoShutdown);
 void back();
 void timeOutChanged(int newTimeOut);
 void saveKnoppixConfiguration();
 void buildPrinterDriverDatabase();
 void deleteLogFiles();
 void raiseWidget(int);
 void refreshRequest(); 
 void send();
 void showIniLetter();
 
public slots: // Public slots
  /**
  * starts dialog for creating a new user
  */
  void slotCreateUser();

  /**
  * Opens a Dialog to change the used HBCI version
  */
  void slotChangeHbci();

  /**
  * retrieves the system id of the bank server 
  */
  void slotGetSystemId();

  /**
  * retrieves a list of our accounts 
  */
  void slotGetAccounts();
  /**
  * sends our own public keys to the bank server 
  */
  void slotSendKeys();
  /**
  * generates the customers iniletter 
  */
  void slotGenerateIniLetter();
  /** Run a test connection to www.google.de
  */
  void slotConnectionTest();
  /** Start the hardware setup dialog
  */
  void slotConsole();
  /** Emits signal help request.
  */
  void slotHelp();
  /** Emits signal importKey().
  */
  void slotImportRDH2Key();

 /** Emits signal sqlquery().
  */
  void slotSQLQuery();
  /** Fires signal setShutdownMode
  */
  void slotShutdownMode(bool autoShutdown);
  /** Emits signal back()
  */
  void slotBack();
  /** Fires signal timeOutChanged
  */
  void slotTimeOutChanged();
  /**
  * slot which retrieves the bank servers public keys 
  */
  void slotGetKeys();
  /** Saves the knoppix configuration
  */
  void slotSaveKnoppixConfiguration();
  /** Fires signal buildPrinterDriverDatabase
  */
  void slotBuildPrinterDriverDatabase();
  /** Emits deleteLogFiles
  */
  void slotDeleteLogFiles();
  
private:
  HoBIT* _hobit;
 /**
  * updates the list of users 
  */
  void updateList();
  GuiCreateUser* _guicreateuser;
  //GuiIniLetter* _guiiniletter;
  GuiHbciVersion* _guichangeversion;
  
 // process to be executed
  QProcess* _process;    
  
};

#endif
