/***************************************************************************
 *   Copyright (C) 2004 by Marcel Mulch, Walter Roth 2005                                    *
 *   marcel@linux                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guitransactioninfo.h"
#include <qdatetime.h>
#include <qlabel.h>
#include <qstring.h>

GuiTransactionInfo::GuiTransactionInfo(QWidget * parent,  GuiMain *main, char * name, HoBIT* hobit)
 : GuiTransactionInfoBase(parent, name){
	_main = main;  
	_hobit = hobit;
   qDebug("Constructor GuiTransactionInfo");   
}


GuiTransactionInfo::~GuiTransactionInfo()
{
  qDebug("Destructor GuiTransactionInfo");
}

void GuiTransactionInfo::slotBack(){
emit goBack();
}

void GuiTransactionInfo::showStandingOrder(Pointer<StandingOrder> sto){
   
   labelLast1->show();
   labelLast2->show();
   labelThree1->show();
   labelThree2->show();
   labelOurCountry1->show();
   labelOurCountry2->show();
   labelOtherCountry1->show();
   labelOtherCountry2->show();

   
   textLabelType->setText("<h3><p align=\"center\">" + tr("Standing Order") +  "</p></h3>");
   textLabelPrimanota->hide();
   labelPrimanota->hide();
   
   clearAll();

   // own account
  // labelOurName->setText(sto.ref().name().c_str());
   labelOurAccount2->setText(sto.ref().accountId().c_str());
   labelOurName2->setText(_OurName);
   labelOurBankId2->setText(sto.ref().bankCode().c_str());
   labelOurCountry2->setText(QString::number(sto.ref().country()));
   
   // other account
   labelOtherAccount2->setText(sto.ref().otherAccountId().c_str());
   labelOtherName2->setText(sto.ref().otherName().c_str());
   labelOtherBankId2->setText(sto.ref().otherBankId().c_str());
   labelOtherCountry2->setText(QString::number(sto.ref().otherCountry()));
   
   
   textLabelAmount->setText(sto.ref().value().toReadableString().c_str());
   labelThree1->setText(tr("Order Number"));
    QString jobid = sto.ref().jobId().c_str();
   labelThree2->setText(QString::number(jobid.toInt()));
   
   Date d = sto.ref().firstDate();
   QDate dateFirst(d.year(), d.month(), d.day());
   d = sto.ref().lastDate();
   QDate dateLast(d.year(), d.month(), d.day());

   labelFirst1->setText(tr("First Execution"));
   labelFirst2->setText(dateFirst.toString( Qt::LocalDate));
   labelLast1->setText(tr("Last Execution"));
   labelLast2->setText(dateLast.toString( Qt::LocalDate));
   
   QString purpose = makePurposeString(sto.ref().purposes());
   textLabelPurpose->setText(purpose);

}


void GuiTransactionInfo::showDatedTransfer(Pointer<DatedTransfer> dt){

   labelLast1->hide();
   labelLast2->hide();
   labelThree1->hide();
   labelThree2->hide();
   labelOurCountry1->show();
   labelOurCountry2->show();
   labelOtherCountry1->show();
   labelOtherCountry2->show();
   
   textLabelType->setText("<h3><p align=\"center\">" + tr("Dated Transfer") +  "</p></h3>");
   textLabelPrimanota->hide();
   labelPrimanota->hide();
   
   clearAll();

   // own account
   labelOurAccount2->setText(dt.ref().accountId().c_str());
   labelOurName2->setText(_OurName);
   labelOurBankId2->setText(dt.ref().bankCode().c_str());
   labelOurCountry2->setText(QString::number(dt.ref().country()));
   
   // other account
   labelOtherAccount2->setText(dt.ref().otherAccountId().c_str());
   labelOtherName2->setText(dt.ref().otherName().c_str());
   labelOtherBankId2->setText(dt.ref().otherBankId().c_str());
   labelOtherCountry2->setText(QString::number(dt.ref().otherCountry()));
     
   textLabelAmount->setText(dt.ref().value().toReadableString().c_str());
   labelThree1->setText(tr("Order Number"));
    QString jobid = dt.ref().jobId().c_str();
   labelThree2->setText(QString::number(jobid.toInt()));
   
   Date d = dt.ref().date();
   QDate date(d.year(), d.month(), d.day());

   labelFirst1->setText(tr("Execution Date"));
   labelFirst2->setText(date.toString( Qt::LocalDate));
   
   QString purpose = makePurposeString(dt.ref().purposes());
   textLabelPurpose->setText(purpose);

}


void GuiTransactionInfo::showTransaction(Pointer<Transaction> trc){
   labelLast1->show();
   labelLast2->show();
   labelThree1->show();
   labelThree2->show();
   labelOurCountry1->hide();
   labelOurCountry2->hide();
   labelOtherCountry1->hide();
   labelOtherCountry2->hide();

   textLabelType->setText("<h3><p align=\"center\">" + tr("Transaction:") + " " + QString(trc.ref().text().c_str()) +  "</p></h3>");
   textLabelPrimanota->show();
   labelPrimanota->show();
   
   clearAll();
   
    // own account
   labelOurAccount2->setText(trc.ref().ouraccountId().c_str());
   labelOurName2->setText(_OurName);
   labelOurBankId2->setText(trc.ref().ourbankcode().c_str());
   
   //labelOurCountry2->setText(QString::number(trc.ref().ourcountry()));
   
   // other account
   labelOtherName2->setText(trc.ref().othername().c_str());
   labelOtherAccount2->setText(trc.ref().otheraccountId().c_str());
   labelOtherBankId2->setText(trc.ref().otherbankcode().c_str());
   
   //labelOtherCountry2->setText(QString::number(trc.ref().othercountry()));

   QString purpose = makePurposeString(trc.ref().purposeLines());
   textLabelPurpose->setText(purpose);
   
   Date d = trc.ref().date();
   QDate date1(d.year(), d.month(), d.day());
   d = trc.ref().storeDate();
   QDate date2(d.year(), d.month(), d.day());  
   d = trc.ref().valutaDate();
   QDate date3(d.year(), d.month(), d.day());  
       
   labelFirst1->setText(tr("Booking Date"));
   labelFirst2->setText(date1.toString( Qt::LocalDate));
   labelLast1->setText(tr("Store Date"));
   labelLast2->setText(date2.toString( Qt::LocalDate));
   labelThree1->setText(tr("Valuta Date"));
   labelThree2->setText(date3.toString( Qt::LocalDate));
   
   textLabelPrimanota->setText(trc.ref().primanota().c_str());
   textLabelAmount->setText(trc.ref().value().toReadableString().c_str());
   
}


/**
*Set Current used Account to Label in Formular 
*/
void GuiTransactionInfo::refresh()
{
	Pointer<Account> account;
        
        if (_main->getCurrentAccount() != "") {
	account = _hobit->findAccount(_main->getCurrentAccount().ascii());
	
	if (account.isValid()){
             _OurName = account.ref().userName().c_str();
	}
        else _OurName = tr("No username found");
	}
	labelOurName2->setText(_OurName);
}

/**
* Clears all textfields
*/
void GuiTransactionInfo::clearAll()
{
// own account
   labelOurAccount2->setText("");
   labelOurBankId2->setText("");
   labelOurCountry2->setText("");
   
   // other account
   labelOtherAccount2->setText("");
   labelOtherAccount2->setText("");
   labelOtherBankId2->setText("");
   labelOtherCountry2->setText("");
    
   textLabelPurpose->setText("");
      
   labelFirst2->setText("");
   labelLast2->setText("");
   labelThree2->setText("");
   
   textLabelPrimanota->setText("");
   textLabelAmount->setText("");
	
}


/** creates purpose lines */
QString GuiTransactionInfo::makePurposeString(list<string> purposeLines){

 QString purpose;

 list <string>::iterator it;

 for (it = purposeLines.begin(); it != purposeLines.end(); it++){
          if(!(*it).empty()) {
	  	purpose+= QString((*it).c_str()) + "\n";
	  }
     }

  purpose.truncate(purpose.length() -1); //remove last \n
  return purpose;
}
