/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUITRANSFERFORM_H
#define GUITRANSFERFORM_H

#include <guitransferformbase.h>

#include <qwidget.h>
#include <qwidgetstack.h>
#include <qvaluevector.h>


#include "hobit.h"
#include <qwidgetstack.h>
#include <guitransferformbase.h>
#include "guimain.h"

class GuiMain;

/**
* Base class for all transfer formulars
*/
class GuiTransferForm : public GuiTransferFormBase
{
  Q_OBJECT
public:

  /**
  *constructor of GuiSingleTransfer
  *
  *@param main a pointer to moneypenny´s main class
  *@param parent a pointer on the main widgets widget stack
  *@param name widgets title
  *@param hobit the hobit object, needed for hbci functions
  */
  GuiTransferForm(QWidget *parent, const char *name=0, HoBIT* hobit = 0);

  /**
  *destructor of GuiSingleTransfer
  */
  virtual ~GuiTransferForm();

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }
  /**
  *Set Current used Account to Label in Formular 
  */
  void refresh();

signals:
 void cancelled();
 void refreshRequest();
 void templateRequest(int Type);
 void sendNow();
 void raiseWidget(int PageId);
   

public slots:

  /**
  *called function after pressing button of formular,
  *gets current inputs of formular for transferjob  
  */
  void accept();
   
  /**
  * Saves the current transfer as a transfer template
  */ 
  void saveTemplate();
  
  
  /**
  * Loads a template from the template list
  */ 
  void fromTemplate();
  
  /**
  *called on reject, emit signal cancelled()
  */
  void reject();
  /** Clears all edits.
  */
  void slotClearForm();

  /**
  *validation of lineEdit name of receiver, 
  *opens messagebox if validation is false
  */
  virtual void validateInputOtherName();

  /**
  *validation of lineEdit accountId, 
  *opens messagebox if validation is false
  */
  virtual void validateInputOtherAccountId();

  /**
  *validation of lineEdit bankCode, 
  *opens messagebox if validation is false 
  */
  virtual void validateInputOtherBankCode();

  /**
  *validation of lineEdit Purpose1, 
  *opens messagebox if validation is false
  */
  virtual void validateInputPurpose();

  /**
  *validation of lineEdit Purpose1, 
  *opens messagebox if validation is false
  */
  virtual void validateInputPurpose2();
  
  /** Called on scrollbar valueChanged
  */
  void slotSliderChanged(int value);
  
  /** Sets _PurposeLines and purpose edits
  */
  void setPurpose(QStringList purpose);
  /** Scrolls the purpose lines upwards
  */
  void slotNextLine();
  /** Scrolls the purpose lines downwards
  */
  void slotPrevLine();

protected:
 /** Checks for empty fields, BLZ for 8 digits and value for !=0.0
 */
 bool isOk();
 /** Creates the job and reurns true on success. To be overwritten.
  */
 virtual bool createJob();
  /** Validates purpose string. Only DTAUS characters are allowed here.
  */
 virtual  bool validatePurpose(QString purpose, QString & _LastInput);
 /** updates the purpose list and returns number of purpose lines.
 */
 virtual int updatePurposeList();
 /** Asks the user for reconfirmation. Returns true, if user clicks yes.
 */
 virtual int userReconfirmation();
 /** Builds the html table rows for the reconfirmation dialog. Returns a string that ist to be included into <table> </table tags. 
 */
 virtual QString reconfirmationTableRows();
 /** Updates the editor's text and takes care of cursor position in
  *  replaced text.
  */
 void updateEditor( QLineEdit * edit, const QString & newText);
  
  HoBIT* _hobit;
  // help-vars for input of lineEdits referring to validation
  QString _LastInputOtherName;
  QString _LastInputOtherAccountId;
  QString _LastInputOtherBankCode;
  QString _LastInputValue;
  QString _LastInputPurpose;
  QString _LastInputPurpose2;
  list<HoBIT_Bank> _banks;
  QDate   _FirstDate;
  QDate   _LastDate;
  
  /**Fields for createJob
   */
  QString _OurAccount;    //DO NOT strip any leading zeros here!   
  QString _OtherName;
  QString _OtherBankCode;
  QString _OtherAccountId;           
  Value   _Value;
   
  /** Clears the form.
  */
  virtual void clear();
 
 /** The header to be used for the reconfirmation message.
 */
  QString _ReconfirmationHeader;

 /** The type of widget, as specified by the Widget number.
 * Used for setting the radiobuttons  in template widget.
 */
 int _FormType; 
 /**The 14 lines with 27 chars legal for a HBCI purpose. Vector allows scrolling.
 */
 QValueVector<QString> * _PurposeLines;
 
 /** The final purpose list. Contains no blank lines. Call updatePurposeList prior to using this list.
 */
 QStringList _PurposeList;
 /** Flag for blocking validation when scrolling
 */
 bool _IsScrolling;
 /**Flag for blocking data reset after saving a template.
  */
 bool _TemplateCreated; 
};

#endif
