/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <list>
#include <string>
using namespace std;

#include <openhbci2.h>
#include <openhbci2/value.h>
using namespace HBCI;

#include <qlistview.h>
#include <qstring.h>
#include <qradiobutton.h>
#include <qpushbutton.h>

#include "guitransfertemplates.h"
#include "hobit_transfertemplate.h"
#include "listviewitems.h"
#include "globals.h"

GuiTransferTemplates::GuiTransferTemplates(QWidget* parent, const char* name, HoBIT* hobit)
: GuiTransferTemplatesBase(parent,name)
{
 _hobit = hobit;
  buttonDelete->setEnabled(false);
  buttonUse->setEnabled(false);
  refresh();
  listViewTemplates->setColumnAlignment(3, Qt::AlignRight);
  listViewTemplates->setColumnWidthMode(4, QListView::Maximum);
}

GuiTransferTemplates::~GuiTransferTemplates()
{
  qDebug("Destructor GuiTransferTemplates finished");
}

/*$SPECIALIZATION$*/
void GuiTransferTemplates::on_listViewTemplates_clicked()
{
  QListViewItem* item = listViewTemplates->selectedItem();
  
  if (item != 0){
    buttonDelete->setEnabled(true);
    buttonUse->setEnabled(true);
  }
  else{
    buttonDelete->setEnabled(false);
    buttonUse->setEnabled(false);
  }
}

void GuiTransferTemplates::on_buttonUse_clicked()
{
   QListViewItem* item; 
   item = listViewTemplates->selectedItem();
   QString name =  item->text(0);
   QString account =  item->text(1);
   QString bank =  item->text(2);
   QString value =  item->text(3);
   QString purposeString = item->text(4);
   QStringList purpose = QStringList::split("\n", purposeString);
   value.truncate(value.length() - 4);
   
   if (radioButtonSingleTransfer->isChecked()){
     emit raiseTemplate(SINGLE_TRANSFER_WIDGET, name, account, bank, value,  purpose);
   }
   if (radioButtonDatedTransfer->isChecked()){
     emit raiseTemplate(DATED_SINGLE_TRANSFER_WIDGET, name, account, bank, value, purpose);
   }
   if (radioButtonStandingOrder->isChecked()){
     emit raiseTemplate(START_STANDING_ORDER_WIDGET, name, account, bank, value, purpose);
   }
}

void GuiTransferTemplates::on_buttonDelete_clicked()
{
   QListViewItem* item = listViewTemplates->selectedItem();
   QString name =  item->text(0);
   QString account =  item->text(1);
   QString bank =  item->text(2);
   QString val = item->text(3);
   val.replace( tr(".", "This is the local decimal point. Dot . in English, Comma , in German"), ".");
   double value =  val.toDouble();
   QString purposeText = item->text(4);
   QStringList purpose = QStringList::split("\n", purposeText);
//   int nLines = purpose.count(); //for debugging

   _hobit->deleteTransferTemplate(_hobit->getCurrentAccount().latin1(), 
                                  name.latin1(), 
                                  account.latin1(), 
                                  bank.latin1(), 
                                  value, 
                                  purpose);
   refresh();
}

void GuiTransferTemplates::cancel()
{
  emit back();
}


void GuiTransferTemplates::refresh()
{  
  _templates = _hobit->getTransferTemplates();
  list<Pointer<HoBIT_TransferTemplate> >::iterator it;
  string currentAccount = HoBIT_String::toIdFormat(_hobit->getCurrentAccount()); //strip any leading zeros
  
  listViewTemplates->clear();
  for(it = _templates.begin(); it != _templates.end(); it++){
     string account = HoBIT_String::toIdFormat((*it).ref().ourAccountId().latin1());//strip any leading zeros
     if (account == currentAccount)
     {
        string name = (*it).ref().otherName().latin1();
        string account = (*it).ref().otherAccountId().latin1();
        string bank = (*it).ref().otherBankCode().latin1();
        double value = (*it).ref().value();
        QString val;
        val.setNum(value, 'f', 2);
        val.replace(".", tr(".", "This is the local decimal point. Dot . in English, Comma , in German"));
        
        //make a string with new lines from stringlist purpose in transfertemplate
        QStringList purpose = (*it).ref().purpose();
//        int nLines = purpose.count(); //for debugging
        QString purposeText = purpose.join("\n");
       // purposeText.stripWhiteSpace(); //remove trailing \n
             
        new HoBIT_ListViewItem(listViewTemplates, //added to and owned by listview automatically
                               name.c_str(),
                               account.c_str(),
                               bank.c_str(),
                               val.latin1(),
                               purposeText);
     }//end if  
  }//end for
  on_listViewTemplates_clicked();  
}

/** set hobit object*/
void GuiTransferTemplates::setHoBIT(HoBIT* hobit){
   _hobit = hobit;
}  


/** Activates the corresponding radiobutton.
*/
void GuiTransferTemplates::slotSetTypeRadioButton(int type){
 switch (type){
  case SINGLE_TRANSFER_WIDGET:         {
                                         radioButtonSingleTransfer->setChecked(true); 
                                         break;
                                       } 
  case DATED_SINGLE_TRANSFER_WIDGET:   {
                                         radioButtonDatedTransfer->setChecked(true); 
                                         break;
                                       } 
  case   START_STANDING_ORDER_WIDGET:  {
                                         radioButtonStandingOrder->setChecked(true); 
                                         break;
                                       }  
  default:                             {
                                         radioButtonSingleTransfer->setChecked(true); 
                                         break;
                                       }  
 }//end switch
}

/**Clears the purpose of the selected template
*/
void GuiTransferTemplates::on_buttonClearPurpose_clicked(){
   QListViewItem* item = listViewTemplates->selectedItem();
   if(!item)
     return;
   QString name =  item->text(0);
   QString account =  item->text(1);
   QString bank =  item->text(2);
   QString val = item->text(3);
   val.replace( tr(".", "This is the local decimal point. Dot . in English, Comma , in German"), ".");
   double value =  val.toDouble();
   QString purposeText = item->text(4);
   QStringList purpose = QStringList::split("\n", purposeText);
   
   _hobit->deleteTransferTemplate(_hobit->getCurrentAccount().latin1(), 
                                  name.latin1(), 
                                  account.latin1(), 
                                  bank.latin1(), 
                                  value, 
                                  purpose);
   HoBIT_TransferTemplate * transferTemplate = 
     new HoBIT_TransferTemplate(_hobit->getCurrentAccount().latin1(), 
                                name.latin1(), 
                                account.latin1(), 
                                bank.latin1(), 
                                value, 
                                "");
   _hobit->addTransferTemplate(transferTemplate);                              
                                
   refresh();                               
}
